#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>

#include "define.h"
#include "capture.h"

/* ====================================================================== */
static void MyError(char *s){
  fprintf(stderr,"Error: %s\n",s);
  exit(1);
}

/* ====================================================================== */
V4L *CaptureInit(char *device, int source){
  V4L   *v4l;

  /*--< V4Lΰ >--*/
  if ((v4l = (V4L *)malloc(sizeof(V4L))) == NULL) MyError("malloc V4L");

  /*--< ǥХΥץ -->*/
  if((v4l->vfd = open(device, O_RDWR))<0) MyError("open device");
 
  /*--< ǥХͤμ >--*/
  if(ioctl(v4l->vfd, VIDIOCGCAP, &v4l->vcap)<0) MyError("VIDIOCGCAP");

  /*--< üҤ >--*/
  v4l->vch.channel=source;
  v4l->vch.norm = VIDEO_MODE_NTSC;
  if (OPT_USB == FALSE)
    if(ioctl(v4l->vfd, VIDIOCSCHAN, &v4l->vch)<0) MyError("VIDIOCSCHAN");

  /*--< Ĵ >--*/
  v4l->vpic.brightness = 32767;
  v4l->vpic.hue        = 32767;
  v4l->vpic.colour     = 32767;
  v4l->vpic.contrast   = 32767;
  v4l->vpic.whiteness  = 32767;
  v4l->vpic.depth      = 24;
  v4l->vpic.palette=VIDEO_PALETTE_RGB24; /* ѥåȷ */
  if(ioctl(v4l->vfd, VIDIOCSPICT, &v4l->vpic)) MyError("VIDIOCSPICT");

  /*--< ȥѥåȤ >--*/
  v4l->vmap.width=WIDTH;
  v4l->vmap.height=HEIGHT;
  v4l->vmap.format=VIDEO_PALETTE_RGB24;

  /*--< mmap μ >--*/
  if(ioctl(v4l->vfd, VIDIOCGMBUF, &v4l->vbuf)<0) MyError("VIDIOCGMBUF");
  if((v4l->mem = mmap(0,v4l->vbuf.size, PROT_READ|PROT_WRITE, MAP_SHARED,
		      v4l->vfd,0))==(char *)-1)
    MyError("mmap");

  return v4l;
}

/* ====================================================================== */
void CaptureSingleFrame(V4L *v4l, char *buffer){
  
  int  n;
  char *ptr;

  v4l->vmap.format=VIDEO_PALETTE_RGB24;

  /* frame #0 ˥ץ㳫ϻؼ */
  v4l->vmap.frame=0;
  if(ioctl(v4l->vfd, VIDIOCMCAPTURE, &v4l->vmap)<0) MyError("VIDIOCMCAPTURE");

  /* frame #0 ץ㽪λԤ */
  if(ioctl(v4l->vfd, VIDIOCSYNC, &v4l->vmap.frame)<0) MyError("VIDIOCSYNC");

  /* RGB줫ʤ饳ԡ */
  ptr = v4l->mem + v4l->vbuf.offsets[v4l->vmap.frame];

  for(n=0; n<WIDTH*HEIGHT; n++) {
    *buffer++=ptr[2];
    *buffer++=ptr[1];
    *buffer++=ptr[0];
    ptr+=3;
  }
}

/* ====================================================================== */
void CaptureSingleFrameGrey(V4L *v4l, char *buffer){
  
  int  n;
  char *ptr;

  v4l->vmap.format=VIDEO_PALETTE_GREY;

  /* frame #0 ˥ץ㳫ϻؼ */
  v4l->vmap.frame=0;
  if(ioctl(v4l->vfd, VIDIOCMCAPTURE, &v4l->vmap)<0) MyError("VIDIOCMCAPTURE");

  /* frame #0 ץ㽪λԤ */
  if(ioctl(v4l->vfd, VIDIOCSYNC, &v4l->vmap.frame)<0) MyError("VIDIOCSYNC");

  /* ԡ */
  ptr = v4l->mem + v4l->vbuf.offsets[v4l->vmap.frame];
  for(n=0; n<WIDTH*HEIGHT; n++) *buffer++ = *ptr++;

}
/* ====================================================================== */
void CaptureSingleFrameAverage(V4L *v4l, char *buffer, int frame){
  int i,j;
  unsigned char *tmp, *tmpP;
  int *sum, *sumP;

  tmp = (char *)malloc(sizeof(unsigned char)*WIDTH*HEIGHT*3);
  sum = (int  *)malloc(sizeof(     int     )*WIDTH*HEIGHT*3);
  for(j=0;j<WIDTH*HEIGHT*3;j++) sum[j]=0;

  for(i=0;i<frame;i++){
    CaptureSingleFrame(v4l, tmp);
    tmpP = tmp;
    sumP = sum;
    for(j=0;j<WIDTH*HEIGHT*3;j++){
      *sumP += *tmpP;
      sumP++;
      tmpP++;
    }
  }
  for(j=0;j<WIDTH*HEIGHT*3;j++) buffer[j]=sum[j]/frame;
  free(tmp);
  free(sum);
}

/* ====================================================================== */
void CaptureSingleFrameGreyAverage(V4L *v4l, char *buffer, int frame){
  int i,j;
  unsigned char *tmp, *tmpP;
  int *sum, *sumP;

  tmp = (char *)malloc(sizeof(unsigned char)*WIDTH*HEIGHT);
  sum = (int  *)malloc(sizeof(     int     )*WIDTH*HEIGHT);
  for(j=0;j<WIDTH*HEIGHT;j++) sum[j]=0;  

  for(i=0;i<frame;i++){
    CaptureSingleFrameGrey(v4l, tmp);
    tmpP = tmp;
    sumP = sum;
    for(j=0;j<WIDTH*HEIGHT;j++){
      *sumP += *tmpP;
      sumP++;
      tmpP++;
    }
  }
  for(j=0;j<WIDTH*HEIGHT;j++) buffer[j]=sum[j]/frame;
  free(tmp);
  free(sum);
}



/* ====================================================================== */
void CaptureMultiFrame(V4L *v4l, char *buffer, int frame){
  
  int  i,n;
  char *ptr;

  struct timeval tv;
  int    sec=0, usec=0, pre_sec=0, pre_usec=0;

  /* frame #0 ˥ץ㳫ϻؼ */
  v4l->vmap.frame=0;
  if(ioctl(v4l->vfd, VIDIOCMCAPTURE, &v4l->vmap)<0) MyError("VIDIOCMCAPTURE");

  /* frame #1 ˥ץ㳫ϻؼ */
  v4l->vmap.frame=1;
  if(ioctl(v4l->vfd, VIDIOCMCAPTURE, &v4l->vmap)<0) MyError("VIDIOCMCAPTURE");

  if (OPT_Verbose){
    gettimeofday(&tv, (struct timezone *)NULL);
    pre_sec  = tv.tv_sec;
    pre_usec = tv.tv_usec;
  }

  /*--< ꤵ줿ե졼ʬβ򥭥ץ >--*/
  v4l->vmap.frame=0;
  for (i=0;i<frame;i++){

    /* ץ㽪λԤ */
    if(ioctl(v4l->vfd, VIDIOCSYNC, &v4l->vmap.frame)<0) MyError("VIDIOCSYNC");

    /*--< Ĺ⡼ɤʤ顤󥿡Х֤ɽ >--*/
    if (OPT_Verbose){
      gettimeofday(&tv, (struct timezone *)NULL);
      sec  = tv.tv_sec;
      usec = tv.tv_usec;
      fprintf(stderr, "Interval=%dms, %ds + %03dms\n",
	      ((sec-pre_sec)*1000000+(usec-pre_usec))/1000,
	      sec,
	      usec/1000);
	      
      pre_sec = sec;
      pre_usec = usec;
    }

    /* RGB줫ʤ饳ԡ */
    ptr = v4l->mem + v4l->vbuf.offsets[v4l->vmap.frame];

    for(n=0; n<WIDTH*HEIGHT; n++) {
      *buffer++=ptr[2];
      *buffer++=ptr[1];
      *buffer++=ptr[0];
      ptr+=3;
    }
    
    /*--< ץ㳫 >--*/
    if (ioctl(v4l->vfd, VIDIOCMCAPTURE, &v4l->vmap)<0) MyError("VIDIOCMCAPTURE");

    /*--< ե졼ֹ촹 >--*/
    v4l->vmap.frame = 1-v4l->vmap.frame;
  }
}


/* ====================================================================== */

void CaptureEnd(V4L *v4l){
  close(v4l->vfd);
  free(v4l);
}
/* ====================================================================== */
