#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

#include "define.h"
#include "capture.h"

int main(int argc, char *argv[]){

  V4L  *v4l;
  int  i, source, frame;
  int  sec, presec;
  char *device="/dev/video0";
  char *captureBuffer;
  FILE *fp;
  char *format, filename[256];
  struct timeval tv;

  int           ch;
  extern char   *optarg;
  extern int    optind;

  /*--<  >--*/
  source = 2;  // Sü
  frame = 1;
  format = (char *)NULL;
  OPT_Verbose = FALSE;
  OPT_TeoMode = FALSE;
  OPT_Wait = -1;
  OPT_Average = 1;
  OPT_USB = FALSE;
  /*--< ץ >--*/
  while ((ch = getopt(argc, argv, "d:s:tm:f:vw:a:u")) != -1){
    switch (ch){
    case 'd':
      device = optarg;
      break;
    case 's':
      source = atoi(optarg);
      break;
    case 't':
      OPT_TeoMode = TRUE;
      break;
    case 'm':
      frame = atoi(optarg);
      break;
    case 'f':
      format = optarg;
      break;
    case 'v':
      OPT_Verbose = TRUE;
      break;
    case 'w':
      OPT_Wait = atoi(optarg);
      break;
    case 'a':
      OPT_Average = atoi(optarg);
      break;
    case 'u':
      OPT_USB = TRUE;
      break;
    default:
      fprintf(stderr, VERSION);
      fprintf(stderr, " -d#: Set device (default:/dev/video0)\n");
      fprintf(stderr, " -s#: Set input source(0-2) (default:2)\n");
      fprintf(stderr, " -t : Output TEO image (default:ppm)\n");
      fprintf(stderr, " -m#: Multi-frame (#: the number of frames)\n");
      fprintf(stderr, " -f#: Set filename format for Multi-frame mode\n");
      fprintf(stderr, " -v : Verbose mode\n");
      fprintf(stderr, " -w#: Wait until gettimeofday(sec.) becomes #\n");
      fprintf(stderr, " -a#: Average of #-frames\n");
      fprintf(stderr, " -u : USB Camera mode (don't check NTSC)\n");
      exit(1);
    }
  }

  /*--< -a  -f Ʊ˻Ǥʤ >--*/
  if ((OPT_Average>1)&&(frame>1)){
    fprintf(stderr,"Error: Options -a and -f are not specified simultaneously.\n");
    exit(1);
  }
  
  /*--< ޥե졼PPMɸϤˤϽ񤱤ʤ >--*/
  if ((OPT_TeoMode==FALSE)&&(frame>1)&&(format==NULL)){
    fprintf(stderr,"Error: if you want to capture multi frame PPM file,\n");
    fprintf(stderr,"       specify filename format by -f option.\n");
    exit(1);
  }
  
  /*--< Video4Linux ν >--*/
  v4l = CaptureInit(device, source);

  /*--< Ǽ >--*/
  if ((captureBuffer=(char *)malloc(sizeof(char)*FSIZE*frame))==NULL){
    fprintf(stderr,"Error:Can't allocate memory\n");
    exit(1);
  }

  /*--< ץ㳫ϻޤԵ >--*/
  if (OPT_Wait > 0) {
    presec=0;
    while(1){
      gettimeofday(&tv, (struct timezone *)NULL);
      sec = tv.tv_sec;
      if (sec == OPT_Wait) break;
      if (sec > OPT_Wait) {
	fprintf(stderr,
		"Error: Now time is %d. The specified time(%d) is past.\n",
		sec, OPT_Wait);
	exit(1);
      }

      if (sec != presec){
	fprintf(stderr, "<%3dsec>\n", OPT_Wait - sec);
	fflush(stderr);
	presec = sec;
      }
    }
  }

  /*--< 󥹤򥭥ץ㤹 >--*/
  // captureBuffer frameǻꤷ֤Ǽ
  if (frame > 1) CaptureMultiFrame(v4l, captureBuffer, frame);
  else CaptureSingleFrameAverage(v4l, captureBuffer, OPT_Average);

  /*--< Ϥ >--*/
  // νˡϡ
  //   եޥåȤϡPPM  TEO 
  //   󥰥ե졼(Ż߲)ޥե졼(ư)
  //   ɸϤ˽񤯤ꤵ줿ե˽񤯤
  // Σ̤Ȥ߹碌롥
  // (PPM & ޥե졼 & ɸ) ϤꤨʤΤǣ̤ꡥ

  if (format == NULL){
    // ɸϤ˽񤭽Ф
    // ppmʤ饷󥰥ե졼˳ꡥTEOʤޥե졼ǽ
    if (OPT_TeoMode == FALSE) printf("P6\n%d %d\n255\n", WIDTH, HEIGHT);
    else  printf("TEO 1\n%d %d 0 0\nU 8\n3 %d\n", WIDTH, HEIGHT, frame);
    fwrite(captureBuffer, sizeof(char), FSIZE*frame, stdout);

  } else {
    // ե˽񤭽Ф(ޥե졼Ϣ֤)
    for (i=0; i<frame; i++){
      if (frame == 1) sprintf(filename,format);
      else  sprintf(filename,format,i);
      if ((fp = fopen(filename,"w"))==NULL){
	fprintf(stderr,"Error:Can't create file(%s)\n",filename);
	exit(1);
      }

      if (OPT_TeoMode == FALSE) fprintf(fp,"P6\n%d %d\n255\n", WIDTH, HEIGHT);
      else fprintf(fp,"TEO 1\n%d %d 0 0\nU 8\n3 1\n", WIDTH, HEIGHT);
      fwrite(captureBuffer + FSIZE*i, sizeof(char), FSIZE, fp);
      fclose(fp);
    }
  }

  /*--< ޤ >--*/
  CaptureEnd(v4l);

  free(captureBuffer);
  return 0;
}
